// 引用三方类库
let MIAOJU_SSKJSDK = require("./tdanalytics.mg.wx.min.js");
import GravityEngine from "./gravityengine.mg.wx.min.js";
let MIAOJU_YLYQSDK;

// 全局常量
const MIAOJU_SDK_VERSION = '1.3.1';
const MIAOJU_PLATFORM_ID = 100001;
const MIAOJU_SS_SERVER_URL = "https://sskj-api.m-co.cn";

// 缓存key
const CACHE_LAUNCH_OPTIONS = "miaoju_launch_options";
const CACHE_CLIENT_ID = "miaoju_client_id";
const CACHE_USER_INFO = "miaoju_user_info";
const CACHE_ROLE_INFO = "miaoju_role_info";

// 返回值code
const RETURN_CODE_SUCCESS = 1000;
const RETURN_CODE_PARAMS_EMPTY = 1001;
const RETURN_CODE_PARAMS_ERROR = 1002;
const RETURN_CODE_LOGIN_ERROR = 1003;
const RETURN_CODE_SYSTEM_ERROR = 1004;
const RETURN_CODE_BUTTON_ERROR = 1005;
const RETURN_CODE_ROLE_ERROR = 1006;
const RETURN_CODE_CUSTOMER_ERROR = 1007;
const RETURN_CODE_PAY_ERROR = 1008;
const RETURN_CODE_USERINFO_ERROR = 1009;
const RETURN_CODE_NOT_INITED = 1010;

// 全局变量
let MIAOJU_GAME_ID = 0;
let MIAOJU_SSAPPID = "";
let MIAOJU_YLYQACCESSTOKEN = "";
let MIAOJU_YLYQTENCENTSDKCONF = {
  // 数据源ID，数字，必填
  "user_action_set_id": '',
  // 加密key，必填
  "secret_key": '',
  // 微信小游戏APPID，wx开头，必填
  "appid": '',
  "silentPeriod": 7
};
let MIAOJU_GAME_VERSION = "";
let MIAOJU_USERINFO_BUTTON;
// 环境变量production为正式环境
let MIAOJU_ENV_CONFIG = "production";
let MIAOJU_SYSTEMINFO = {};
let MIAOJU_DEVICEINFO = {};
let MIAOJU_LAUNCHOPTIONS = {};
let MIAOJU_IS_INITED = false;

// 全局激励视频广告对象
let MIAOJU_REWARDVIDEOAD;
let MIAOJU_REQUEST_HOST = {
    production: 'https://minigamesdk-api.miaorui1.com',
    test: 'https://pre-minigamesdk-api.miaorui1.com'
};
const IS_MOCK = false


/**
 * 初始化
 * @params
 *  gameId SDK游戏id
 *  gameVersion 游戏版本号
 *  ssAppId 数数appid
 *  ylyqAccessToken 引力引擎access
 *  envConfig 环境变量 production/test
 * @callback 回调方法名
 */
function init(params, callback = () => {}) {
    /*params = {
         "gameId": 1001,
         "gameVersion": "v1.0.0", //游戏版本号
         "ssAppId": "822bb3ff1009407ebd76ed1787717c54",
         "ylyqAccessToken": "IlmBjXF4dngy7c3amghAic6utl1EbHdy",
         "tencentSdkData": {
            // 数据源ID，数字，必填
            "user_action_set_id": 1207341440,
            // 加密key，必填
            "secret_key": '3e94210352fc5d69c388da836a2fa168',
            // 微信小游戏APPID，wx开头，必填
            "appid": 'wx8f8a79d64b59d5f2',
            // 
            "silentPeriod": 7
        },
         "envConfig": "test"
    };*/
    consolePrintWithTime("init reciveparam:", params);
    consolePrintWithTime("init callback:", callback);

    // 判断参数是否正确
    if (!params.gameId) {
        callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "gameId missing", {}));
        return false;
    }
    if (!params.gameVersion) {
        callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "gameVersion missing", {}));
        return false;
    }
    if (!params.ssAppId) {
        callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "ssAppId missing", {}));
        return false;
    }
    if (!params.ylyqAccessToken) {
        callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "ylyqAccessToekn missing", {}));
        return false;
    }
    if (params.envConfig) {
        MIAOJU_ENV_CONFIG = params.envConfig;
    }
    MIAOJU_GAME_ID = params.gameId;
    MIAOJU_SSAPPID = params.ssAppId;
    MIAOJU_YLYQACCESSTOKEN = params.ylyqAccessToken;
    MIAOJU_GAME_VERSION = params.gameVersion;
    if (!params.tencentSdkData) {
      MIAOJU_YLYQTENCENTSDKCONF = params.tencentSdkData
    }
    //consolePrintWithTime("init options:",params.gameId,params.ssAppId,MIAOJU_GAME_ID,MIAOJU_SSAPPID);
    // 获取启动参数 系统函数调用
    let options = getOptions(0);
    consolePrintWithTime("init options:", options);

    //群入口进来判断
    if(options.scene == 1008){

    }

    // 获取系统信息
    let systemInfo = wx.getSystemInfoSync();
    let windowInfo = wx.getWindowInfo();
    let appInfo = wx.getAppBaseInfo();
    consolePrintWithTime("init systemInfo:", systemInfo, windowInfo, appInfo);

    MIAOJU_SYSTEMINFO = {
        "brand": systemInfo.brand,
        "model": systemInfo.model,
        "system": systemInfo.system,
        "platform": systemInfo.platform, // ios/android/windows/mac/devtools
        "memorySize": systemInfo.memorySize,
        "screenWidth": windowInfo.screenWidth,
        "screenHeight": windowInfo.screenHeight,
        "SDKVersion": appInfo.SDKVersion,
        "language": appInfo.language,
        "version": appInfo.version
    }
    MIAOJU_DEVICEINFO = {
        "brand": systemInfo.brand,
        "model": systemInfo.model,
        "version": appInfo.version,
        "system": systemInfo.system,
        "scene": options.scene,
        "sdk_version": MIAOJU_SDK_VERSION,
        "game_version": MIAOJU_GAME_VERSION
    }
    // 获得网络类型
    wx.getNetworkType({
        success(res) {
            MIAOJU_DEVICEINFO.network = res.networkType
        }
    })

    consolePrintWithTime("init deviceinfo:", MIAOJU_DEVICEINFO);

    //获取客户端唯一id
    let clientId = getClientId();
    consolePrintWithTime("init clientId:", clientId);

    //数数初始化
    // TA SDK 配置对象
    var ssConfig = {
        appId: MIAOJU_SSAPPID, // 项目的 APP ID
        serverUrl: MIAOJU_SS_SERVER_URL, // 数据上报地址
        autoTrack: {
            appShow: true, // 自动采集 ta_mg_show
            appHide: true // 自动采集 ta_mg_hide
        }
    };
    consolePrintWithTime("init ssConfig:", ssConfig);
    // 初始化
    MIAOJU_SSKJSDK.init(ssConfig);
    //设置数数访客id
    MIAOJU_SSKJSDK.setDistinctId(clientId);
    // 设置当前设备唯一识别
    ssUserSet({
        type: 1,
        properties: {
            mj_unique_ID: {
                mj_unique_ID: clientId
            }
        }
    })

    // 请求初始化接口 /init
    apiRequest({
        url: MIAOJU_REQUEST_HOST[MIAOJU_ENV_CONFIG] + '/init',
    }).then((res) => {
        if (IS_MOCK) {
            res = {
                "code": 1000,
                "message": "success",
                "data": {
                    "client_id": "or4olsdrflwee4lj23r4lwf", //设备唯一标识原样返回
                    "create_time": "2018-01-01 01:23:45" // 设备激活时间 精确到秒或毫秒
                }
            }
        }
        const {
            code,
            data
        } = res
        if (code === 1000) {
            ssUserSet({
                type: 2,
                properties: {
                    mj_device_active_time: data.create_time || getTime()
                }
            })
        }
    }, (err) => {})
    // 微信登录获取登录状态和信息
    login((res) => {
        consolePrintWithTime("init login res:", res);
        if (res.code == 1000) {
            // 登录成功并获取到用户信息
            const userData = res.data;
            const platformUserId = userData.platform_user_id;
            //初始化引力引擎
            const ylyqConfig = {
                accessToken: MIAOJU_YLYQACCESSTOKEN, // 项目通行证，在：网站后台-->设置-->应用列表中找到Access Token列 复制（首次使用可能需要先新增应用）
                clientId: platformUserId, // 用户唯一标识，如产品为小游戏，则必须填用户openid（注意，不是小游戏的APPID！！！）
                autoTrack: {
                    appLaunch: true, // 自动采集 $MPLaunch
                    appShow: true, // 自动采集 $MPShow
                    appHide: true, // 自动采集 $MPHide
                },
                name: "MIAOJU_YLYQSDK", // 全局变量名称, 默认为 gravityengine
                debugMode: MIAOJU_ENV_CONFIG == "production" ? "none" : "debug", // 是否开启测试模式，开启测试模式后，可以在 网站后台--设置--元数据--事件流中查看实时数据上报结果。（测试时使用，上线之后一定要关掉，改成none或者删除）
                tencentSdkData: MIAOJU_YLYQTENCENTSDKCONF
              };
            MIAOJU_YLYQSDK = new GravityEngine(ylyqConfig);
            MIAOJU_YLYQSDK.setupAndStart();
            // 请求初始化接口
            MIAOJU_YLYQSDK.initialize({
                    name: platformUserId,
                    version: 1,
                    openid: platformUserId,
                    enable_sync_attribution: false, // 是否开启同步获取归因信息，具体请参考同步归因：https://doc.gravity-engine.com/turbo-integrated/sync_attribution.html
                })
                .then((res) => {
                    consolePrintWithTime("ylyq initialize success " + res);
                })
                .catch((err) => {
                    consolePrintWithTime("ylyq initialize failed, error is " + err);
                });
            // 引力引擎打开游戏 
            ylyqReport({
                eventName: 'gravity_game_open'
            })
            // 设置用户属性
            ssUserSet({
                type: 1,
                properties: {
                    mj_last_login: userData.last_login_time
                }
            })
            if (userData.is_new == 1) {
                // 如果是新用户的话上报注册
                ssUserSet({
                    type: 2,
                    properties: {
                        mj_register_time: userData.create_time || getTime()
                    }
                })
                // 数数创建账号上报
                ssReport({
                    eventName: 'mj_creat_account'
                })
                // 引力引擎 创建账号上报
                MIAOJU_YLYQSDK.registerEvent()
            }
            // 引力引擎 登录账号上报
            MIAOJU_YLYQSDK.loginEvent()
            // 数数登录账号上报 ?? 
            ssReport({
                eventName: 'mj_login_account',
                properties: {
                    mj_login_method: '微信小游戏'
                }
            })
            // 初始化分享菜单
            wx.showShareMenu({
                withShareTicket: true,
                menus: ['shareAppMessage', 'shareTimeline']
            });
            // 用户信息
            let userInfo = {
                // "nickName": userData.nick_name,
                "platformUserId": platformUserId, // 其实是open_id
                "sdkUserId": userData.user_id || 0, // sdk端的用户id
                "clientId": clientId, //访客id
                "isNew": userData.is_new // 0非新用户，1新用户
            };
            // 设置已经初始化完成了
            MIAOJU_IS_INITED = true;
            callback(getSuccessReturn({
                "userInfo": userInfo,
                "launchOptions": options,
                "systemInfo": MIAOJU_SYSTEMINFO
            }));
        } else {
            callback(getReturn(RETURN_CODE_LOGIN_ERROR, "init login error:" + res.message, {}));
            return false;
        }
    });
}
/**
 * 登录
 * @callback 回调方法名
 */
function login(loginCallback) {
    // 先使用sdk登录态登录，如果失败则重新获取微信code登录
    // 如果没有token直接去获取code
    const user_info = wx.getStorageSync(CACHE_USER_INFO)
    const token = user_info.token || ''
    consolePrintWithTime("token缓存：", token);

    if (!token) {
        // 没有token 去wx.login获取code
        userLoginWithWx(loginCallback,"tokenEmpty");
    } else {
        // 有token 先检测下微信session_key有没有过期，过期则还是要调code登录，否则用token去请求
        wx.checkSession({
          success () {
            //session_key 未过期，并且在本生命周期一直有效
            userLogin({
                token: token
            }, loginCallback)
          },
          fail () {
            // session_key 已经失效，需要重新执行登录流程
            userLoginWithWx(loginCallback,"checkSessionFail");
          }
        })
        
    }
}

//微信code登录
function userLoginWithWx(loginCallback,fromStr=''){
  wxPromisify(wx.login)({
    timeout: 20000,
  }).then(
    (res) => {
        const {
            code
        } = res
        if (code) {
            consolePrintWithTime("[userLoginWithWx]"+fromStr+" 这里获取到code:", code);
            // 获取到code 去请求用户登录接口
            userLogin({
                code: code
            }, loginCallback)
        } else {
            console.error("[userLoginWithWx]"+fromStr+' wx.login获取code失败')
            loginCallback({
                "code": RETURN_CODE_LOGIN_ERROR,
                "message": 'wx.login获取code失败'
            });
        }
    },
    (err) => {
        console.error("[userLoginWithWx]"+fromStr+' wx.login报错', err)
        loginCallback({
            "code": RETURN_CODE_LOGIN_ERROR,
            "message": err.errMsg || 'wx.login error'
        })
    })
}

/**
 * 请求登录接口
 */
function userLogin(params, loginCallback) {
    apiRequest({
        url: MIAOJU_REQUEST_HOST[MIAOJU_ENV_CONFIG] + '/user/login',
        data: params
    }).then(
        (result) => {
            if (IS_MOCK) {
                result = {
                    "code": 1000,
                    "message": "success",
                    "data": {
                        "user_id": 10000001, //SDK用户ID
                        "platform_user_id": "lwoonwmyyyabltlwylvs", //即微信open_id
                        "platform_id": 100001, //平台ID
                        "game_id": 1001, //游戏ID
                        "token": "lwoonwmyyyabltlwylvs", //登录态
                        "nick_name": "张胖", //昵称
                        "gender": "男", //性别：男、女、未知
                        "avatar_url": "https://****", //头像
                        "union_id": "oonwmyyyabltsraslwylvs", //即微信open_id
                        "channl_id": 10000001, //渠道来源
                        "is_new": 1, // 0非新用户，1新用户
                        "create_time": "2018-01-01 01:23:45", // 注册时间
                        "last_login_time": "2018-01-01 01:23:45", // 最后登录时间
                        "total_login_num": 10, // 累计登录次数
                        "active_days": 8, // 累计活跃天数
                    }
                }
            }
            const {
                code,
                message,
                data
            } = result;
            if (code === 1000) {
                // 登录成功 添加用户信息缓存
                wx.setStorageSync(CACHE_USER_INFO, data);
                // 执行登录回调 loginCallback
                loginCallback(result);
            } else if (code === 1001) {
                // token 登录态无效或已过期 清除用户信息（登录态）
                wx.removeStorageSync(CACHE_USER_INFO);
                // 如果是根据token去请求用户登录接口的，则有token失效的风险，所以这个接口会有token失效的概率，如果返回此错误码，且请求参数token不为空，则重试 则重新使用微信登录获取到的code 去请求登录接口
                if (params.token) {
                  userLoginWithWx(loginCallback,"tokenLoginFail");

                } else {
                    loginCallback({
                        "code": code,
                        "message": message
                    });
                }
            } else {
                loginCallback({
                    "code": code,
                    "message": message
                });
            }
        }, (err) => {
            loginCallback({
                "code": RETURN_CODE_LOGIN_ERROR,
                "message": '请求 /login接口错误'
            });
        })
}

/**
 * 支付 
 */
function pay(params, callback = () => {}) {
    if (isInited()) {
        consolePrintWithTime("pay reciveparam:", params);
        consolePrintWithTime("pay callback:", callback);
        // 先创建订单，创建成功=>获得订单号和拉起米大师所需的参数
        // params = {
        //     "productId": "mj_unity_sdk_test", //商品ID
        //     "gameOrderId": "10000123456", //游戏订单ID
        //     "customData": "", //自定义参数
        //     "productName": "MJSDK道具", //商品名称
        //     "serverId": "4", //服ID
        //     "serverName": "5", //服名称
        //     "roleId": "6", //角色ID
        //     "roleName": "7", //角色名
        //     "roleLevel": "8", //角色等级
        //     "buyQuantity": 1, //购买数量
        //     "goodsPrice": 100, //单价分 1元=100分
        //     "payNoticeUrl": "", //支付回调服务端地址
        //     "conversationParams": {
        //         "showMessageCard": true,
        //         "sendMessageTitle": "我是一个充值小程序O O"
        //     } // 要传的客服会话参数
        // };
        // 判断参数是否正确
        if (!params.gameOrderId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "gameOrderId missing", {}));
            return false;
        }
        if (!params.productId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "productId missing", {}));
            return false;
        }
        if (!params.serverId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "serverId missing", {}));
            return false;
        }
        if (!params.serverName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "serverName missing", {}));
            return false;
        }
        if (!params.roleId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleId missing", {}));
            return false;
        }
        if (!params.roleName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleName missing", {}));
            return false;
        }
        if (!params.productName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "productName missing", {}));
            return false;
        }
        if (!params.buyQuantity) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "buyQuantity missing", {}));
            return false;
        }
        if (!params.goodsPrice) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "goodsPrice missing", {}));
            return false;
        }
        const pay_params = {
            game_order_id: params.gameOrderId,
            product_id: params.productId,
            product_name: params.productName,
            buy_quantity: params.buyQuantity,
            goods_price: params.goodsPrice,
            pay_notify_url: params.payNoticeUrl || '',
            custom_data: params.customData || '',
        }
        // 请求创单接口创建订单和米大师支付参数
        apiRequest({
            url: MIAOJU_REQUEST_HOST[MIAOJU_ENV_CONFIG] + '/order/create',
            data: pay_params
        }).then((result) => {
            if (IS_MOCK) {
                result = {
                    "code": 3004,
                    "message": "success",
                    "data": {
                        "order_id": 8866202408031533560001, //订单唯一编号
                        "sign_data": '{"mode":"goods","offerId":"123","buyQuantity":1,"env":0,"currencyType":"CNY","platform":"android","zoneId":"1","productId":"testproductId","goodsPrice":10,"outTradeNo":"xxxxxx","attach":"testdata"}', //本次返回的为字符串类型的JSON格式
                        "pay_sign": "d0b8bbccbe34ed11549bcfd6602b08711f4acc0965253a949cd6a2b895152f9d", //支付签名
                        "signature": "d0b8bbccbe34ed11549bcfd6602b08711f4acc0965253a949cd6a2b895152f9d", //用户态签名
                    }
                }
            }
            const {
                code,
                data,
                message
            } = result;
            if (code == 1000 || code == 3004) {
                // 引力引擎上报 创单
                ylyqReport({
                    eventName: 'gravity_app_order',
                    properties: {
                        pay_reason: params.productName,
                        pay_amount: params.goodsPrice * params.buyQuantity,
                        order_id: data.order_id || ''
                    }
                })
                // 数数上报 创单
                ssReport({
                    eventName: 'mj_order_init',
                    properties: {
                        mj_product_id: params.productId,
                        mj_order_id: data.order_id || '',
                        mj_num: params.goodsPrice * params.buyQuantity/100,
                        mj_order_time: getTime(),
                    }
                })
            }
            if (code === 1000) {
                // 创建订单成功
                const {
                    order_id,
                    sign_data,
                    pay_sign,
                    signature
                } = data;
                // 去拉米大师
                wx.requestMidasPaymentGameItem({
                    signData: sign_data,
                    paySig: pay_sign,
                    signature: signature,
                    success: function (res, errCode) {
                        consolePrintWithTime("requestMidasPaymentGameItem success res:", res, errCode);
                        ylyqReport({
                          eventName: 'Pay',
                          properties: {
                            payAmount: params.goodsPrice * params.buyQuantity,
                            payType:"CNY",
                            orderId: order_id,
                            payReason: params.productName,
                            payMethod: "微信虚拟支付"
                          }
                        });
                        // 支付成功
                        callback({
                            "code": RETURN_CODE_SUCCESS,
                            "message": 'requestMidasPaymentGameItem 支付成功',
                            "data": {
                                "sdkOrderId": order_id || ''
                            }
                        });
                    },
                    fail: function (res) {
                        // 失败
                        consolePrintWithTime("requestMidasPaymentGameItem error res:", res);
                        const {
                            errCode,
                            errMsg
                        } = res;
                        if (errCode === 701001) {
                            // 表示ios禁止支付
                            callback({
                                "code": RETURN_CODE_PAY_ERROR,
                                "message": 'ios禁止支付',
                                "data": {
                                    "sdkOrderId": order_id || ''
                                }
                            });
                        } else {
                            // 其他报错
                            callback({
                                "code": RETURN_CODE_PAY_ERROR,
                                "message": errMsg,
                                "data": {
                                    "sdkOrderId": order_id || ''
                                }
                            });
                        }
                    },
                    complete: function (res) {
                        consolePrintWithTime("requestMidasPaymentGameItem complete res:", res);
                    }
                });
            } else if (code === 3003) {
                // 表示IOS系统暂不支持购买 直接返回
                callback({
                    "code": code,
                    "message": 'IOS系统暂不支持购买',
                    "data": {}
                });
            } else if (code === 3004) {
                // 拉起客服
                wx.showModal({
                    title: '支付提示',
                    content: message,
                    showCancel: false,
                    confirmText: '我知道了',
                    success(res) {
                        if (res.confirm) {
                            wxPromisify(wx.openCustomerServiceConversation)(params.conversationParams || {}).then(
                                (res) => {
                                    consolePrintWithTime("wx.openCustomerServiceConversation success:", res);
                                }, (err) => {
                                    consolePrintWithTime("wx.openCustomerServiceConversation error:", err);
                                })
                        }
                    }
                })
                // 表示IOS系统请前往客服咨询
                callback({
                    "code": code,
                    "message": 'IOS系统请前往客服咨询',
                    "data": {
                        "sdkOrderId": data.order_id || ''
                    }
                });
            } else {
                callback({
                    "code": RETURN_CODE_PAY_ERROR,
                    "message": 'order/create 接口创单失败 ' + message,
                    "data": {}
                });
            }
        }, (err) => {
            callback({
                "code": RETURN_CODE_PAY_ERROR,
                "message": '请求 order/create 接口失败 ' + (err.errMsg || ''),
                "data": {}
            });
        })
    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }
}

/**
 * 初始化并展示激励视频广告
 * @params 
 *  adUnitId 广告id
 * @callback 回调方法名
 */
function rewardVideoAdCreate(params, callback = () => {}) {
    if (isInited()) {
        // params = {
        //     "adUnitId": "adunit-b2c1849f409b5d48",
        //     "otherProperties": {}
        // };
        consolePrintWithTime("rewardVideoAdCreate params:", params);
        consolePrintWithTime("rewardVideoAdCreate callback:", callback);
        if (!params.adUnitId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "adUnitId missing", {}));
            return false;
        }
        // 用户打开广告
        ssReport({
            eventName: 'mj_ad_init'
        })

        // 创建激励视频广告实例，提前初始化
        MIAOJU_REWARDVIDEOAD = wx.createRewardedVideoAd({
            adUnitId: params.adUnitId
        })

        // 广告加载
        MIAOJU_REWARDVIDEOAD.onLoad(() => {
            consolePrintWithTime("rewardVideoAdCreate onLoad:ok");
        })

        // 用户触发广告后，显示激励视频广告
        MIAOJU_REWARDVIDEOAD.load()
            .then(() => {
                MIAOJU_REWARDVIDEOAD.show()
                    .then(() => {
                        MIAOJU_YLYQSDK.adShowEvent("reward", params.adUnitId, params.otherProperties || {});
                    })
                    .catch((err1) => {
                        consolePrintWithTime("rewardVideoAdShow err1", err1);
                        // 失败重试
                        MIAOJU_REWARDVIDEOAD.load()
                            .then(() => {
                                MIAOJU_REWARDVIDEOAD.show().then(() => {
                                    MIAOJU_YLYQSDK.adShowEvent("reward", params.adUnitId, params.otherProperties || {});
                                })
                            })
                            .catch(err2 => {
                                callback(getReturn(RETURN_CODE_PARAMS_ERROR, "rewardVideoAdShow error", err2));
                                return false;
                            })
                    })
            })

        MIAOJU_REWARDVIDEOAD.onError(err => {
            consolePrintWithTime("rewardVideoAdShow onError:", err);
            callback(getReturn(RETURN_CODE_PARAMS_ERROR, "rewardVideoAdShow onError", err));
            return false;
        });

        // 监听用户关闭广告
        MIAOJU_REWARDVIDEOAD.onClose(res => {
            consolePrintWithTime("rewardVideoAdShow onClose:", res);
            // 用户点击了【关闭广告】按钮
            // 小于 2.1.0 的基础库版本，res 是一个 undefined
            if (res && res.isEnded || res === undefined) {
                // 正常播放结束，可以下发游戏奖励 
                // 用户看完广告
                ssReport({
                    eventName: 'mj_ad_finish_done'
                })
                ylyqReport({
                    eventName: 'gravity_ad_view'
                })
                consolePrintWithTime("rewardVideoAdShow onClose:ok");
                callback(getSuccessReturn({}));
                return true;
            } else {
                // 播放中途退出，不下发游戏奖励
                // 用户中途关闭广告（时长未达获奖条件）
                ssReport({
                    eventName: 'mj_ad_finish_quit'
                })
                consolePrintWithTime("rewardVideoAdShow onClose:break");
                callback(getReturn(RETURN_CODE_SYSTEM_ERROR, "rewardVideoAdShow onClose no finish", res));
                return false;
            }
        });
    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }
}

/**
 * 展示激励视频广告（废弃）
 * @params 
 * @callback 回调方法名
 */
function rewardVideoAdShow(params, callback) {
    consolePrintWithTime("rewardVideoAdShow params", params);
}

/**
 * 分享 监听分享 只能监听到分享给朋友的
 * @param {*} params 
 * @param {*} callback 
 */
function share(params, callback = () => {}) {
    if (isInited()) {
        // params = {
        //     "title": "我是一个转发标题!!!",
        //     "imageUrl": "", //转发显示图片的链接，可以是网络图片路径或本地图片文件路径或相对代码包根目录的图片文件路径
        //     "imageUrlId": "", //审核通过的图片 ID,微信后台上传审核
        //     "query": "a=1&b=2", //查询字符串，必须是 key1=val1&key2=val2 的格式。从这条转发消息进入后，可通过 wx.getLaunchOptionsSync() 或 wx.onShow() 获取启动参数中的 query。不传则默认使用当前页面query。SDK
        // }
        consolePrintWithTime("share reciveparam", params);
        consolePrintWithTime("share callback", callback);

        // 判断参数
        if (!params.title) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "title missing", {}));
            return false;
        }
        const user_info = wx.getStorageSync(CACHE_USER_INFO)
        // 需要拼一下返回给游戏的query
        if (params.query != undefined && params.query != '') {
            params.query += "&platform_id=" + MIAOJU_PLATFORM_ID + "&platfrom_user_id=" +
                (user_info.platform_user_id || '')
        } else {
            params.query = "platform_id=" + MIAOJU_PLATFORM_ID + "&platfrom_user_id=" +
                (user_info.platform_user_id || '')
        }
        // 监听转发
        wx.onShareAppMessage(() => {
            consolePrintWithTime("ShareAppMessage share start");

            //上报引力引擎，引力上报腾讯
            ylyqReport({
              eventName: 'Share',
              properties: {
                type:'APP_MESSAGE'
              }
            })

            // 监听到分享，游戏 callback()
            callback({
                code: RETURN_CODE_SUCCESS,
                message: "ShareAppMessage成功",
                data: params
            })
            return params;
        })
        // if (wx.onShareTimeline) {
        //     wx.onShareTimeline(() => {
        //         consolePrintWithTime("onShareTimeline share start");
        //         // 监听到分享，游戏 callback()
        //         callback({
        //             code: RETURN_CODE_SUCCESS,
        //             message: "ShareTimeline成功",
        //             data: params
        //         })
        //         return params;
        //     })
        // }

    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }

}
/**
 * 主动分享
 * @param {*} params 
 * @param {*} callback 
 */
function shareOpen(params, callback = () => {}) {
    if (isInited()) {
        consolePrintWithTime("shareOpen reciveparam", params);
        consolePrintWithTime("shareOpen callback", callback);
        // params = {
        //     "title": "我是一个转发标题!!!",
        //     "imageUrl": "", //转发显示图片的链接，可以是网络图片路径或本地图片文件路径或相对代码包根目录的图片文件路径
        //     "imageUrlId": "", //审核通过的图片 ID,微信后台上传审核
        //     "query": "a=1&b=2", //查询字符串，必须是 key1=val1&key2=val2 的格式。从这条转发消息进入后，可通过 wx.getLaunchOptionsSync() 或 wx.onShow() 获取启动参数中的 query。不传则默认使用当前页面query。SDK
        // }
        // 判断参数
        if (!params.title) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "title missing", {}));
            return false;
        }
        const user_info = wx.getStorageSync(CACHE_USER_INFO)
        // 需要拼一下返回给游戏的query
        if (params.query != undefined && params.query != '') {
            params.query += "&platform_id=" + MIAOJU_PLATFORM_ID + "&platfrom_user_id=" + (user_info.platform_user_id || '')
        } else {
            params.query = "platform_id=" + MIAOJU_PLATFORM_ID + "&platfrom_user_id=" + (user_info.platform_user_id || '')
        }
        // 主动拉起转发，进入选择通讯录界面
        wx.shareAppMessage(params);

        //上报引力引擎，引力上报腾讯
        ylyqReport({
          eventName: 'Share',
          properties: {
            type:'APP_MESSAGE'
          }
        })
        callback({
            code: RETURN_CODE_SUCCESS,
            message: "主动share成功",
            data: params
        });
    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }
}

/**
 * 打开客服
 * @param {*} params 
 * @param {*} callback 
 */
function openCustomerService(params, callback = () => {}) {
    if (isInited()) {
        consolePrintWithTime("openCustomerService reciveparam", params);
        consolePrintWithTime("openCustomerService callback", callback);

        // params = {
        //     "showMessageCard": true,
        //     "sendMessageTitle": "我是一个小游戏标题哦",
        //     "sendMessageImg": "https://pic.leduimg.com/pic-ledu-com/productEnvironment/1SHjaba_1920x780.jpg"
        // }
        wxPromisify(wx.openCustomerServiceConversation)(params || {}).then(
            (res) => {
                consolePrintWithTime("openCustomerServiceConversation res", res);
                callback({
                    code: RETURN_CODE_SUCCESS,
                    message: "openCustomerService成功",
                    data: {}
                });
            }, (err) => {
                consolePrintWithTime("openCustomerServiceConversation err", err);
                callback(getReturn(RETURN_CODE_CUSTOMER_ERROR, "openCustomerService error", {}));
            })
    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }
}

/**
 * 获取用户信息
 * @param {*} params 
 * @param {*} callback 
 */
function getUserInfo(params, callback = () => {}) {
  const funcNameMsg = "getUserInfo";
  if (!isInited()) {
    // 未初始化
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": '未完成初始化',
      "data": {}
    });
  }
  consolePrintWithTime("getUserInfo reciveparam", params,callback);

  const user_params = {
    "UserInfo": {},
    "rawData": "",
    "signature": "",
    "encryptedData": "",
    "iv": ""
  }
  wx.getSetting({
    success(res) {
      consolePrintWithTime(funcNameMsg+" getSetting success:",res)
      if (!res.authSetting['scope.userInfo']) {
        consolePrintWithTime(funcNameMsg+" getSetting no scope.userInfo:")
        reqUserInfo({}, callback);
      }else{
        wxPromisify(wx.getUserInfo)({
          withCredentials: true
        }).then(
          (res) => {
              // 成功获取用户信息 
              user_params.encryptedData = res.encryptedData;
              user_params.rawData = res.rawData;
              user_params.signature = res.signature;
              user_params.iv = res.iv;
              user_params.UserInfo = res.UserInfo;
              reqUserInfo({
                  user_info: JSON.stringify(user_params)
              }, callback);

          }, (err) => {
            // 获取失败,不更新最新的用户信息
            consolePrintWithTime(funcNameMsg+" wx.getUserInfo err:",err)
            reqUserInfo({}, callback);
          })
      }
    }
  })
}

/**
 * 创建授权按钮 获取用户基本信息
 */
function createUserInfoButton(params, callback = () => {}) {
    if (isInited()) {
        consolePrintWithTime("createUserInfoButton reciveparam", params);
        consolePrintWithTime("createUserInfoButton callback", callback);
        // params = {
        //     button: {
        //         type: 'text',
        //         text: '获取用户信息',
        //         style: {
        //             left: 10,
        //             top: 76,
        //             width: 200,
        //             height: 40,
        //             lineHeight: 40,
        //             backgroundColor: '#ff0000',
        //             color: '#ffffff',
        //             textAlign: 'center',
        //             fontSize: 16,
        //             borderRadius: 4
        //         },
        //         withCredentials: true
        //     }
        // }
        if (!params.button) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "button missing", {}));
            return false;
        }
        const user_params = {
            "UserInfo": {},
            "rawData": "",
            "signature": "",
            "encryptedData": "",
            "iv": ""
        }
        wxPromisify(wx.getSetting)().then(
            (res) => {
                const {
                    authSetting
                } = res
                if (authSetting['scope.userInfo'] === true) {
                    // 开启授权了 直接调用  wx.getUserInfo 获取信息
                    wxPromisify(wx.getUserInfo)({
                        withCredentials: true
                    }).then(
                        (res) => {
                            // 成功获取用户信息 
                            user_params.encryptedData = res.encryptedData;
                            user_params.rawData = res.rawData;
                            user_params.signature = res.signature;
                            user_params.iv = res.iv;
                            user_params.UserInfo = res.UserInfo;
                            reqUserInfo({
                                user_info: JSON.stringify(user_params)
                            }, callback);
                        }, (err) => {
                            // 获取失败
                            callback(getReturn(RETURN_CODE_BUTTON_ERROR, "wx.getUserInfo 获取用户信息失败", {}));
                        })
                } else {
                    // 创建授权按钮
                    MIAOJU_USERINFO_BUTTON = wx.createUserInfoButton(params.button);
                    console.log("MIAOJU_USERINFO_BUTTON",MIAOJU_USERINFO_BUTTON);
                    MIAOJU_USERINFO_BUTTON.onTap((res) => {
                        if (res.encryptedData != "" && res.encryptedData != null && res.encryptedData != undefined) {
                            // 如果拿到这些数据，说明一定同意授权
                            // 同意用户信息授权弹窗
                            ssReport({
                                eventName: 'mj_agreement_head',
                                properties: {
                                    mj_agreement_confirm: true
                                }
                            })
                            // 调用用户信息的接口解析用户基本信息
                            // 成功获取用户信息 
                            user_params.encryptedData = res.encryptedData;
                            user_params.rawData = res.rawData;
                            user_params.signature = res.signature;
                            user_params.iv = res.iv;
                            user_params.UserInfo = res.UserInfo;
                            reqUserInfo({
                                user_info: JSON.stringify(user_params)
                            }, callback);
                            //授权后关闭按钮
                            //MIAOJU_USERINFO_BUTTON.destroy();
                            //console.log("MIAOJU_USERINFO_BUTTON des:",MIAOJU_USERINFO_BUTTON);
                            destoryUserInfoButton();
                        } else {
                            // 拒绝用户信息授权弹窗
                            ssReport({
                                eventName: 'mj_agreement_head',
                                properties: {
                                    mj_agreement_refuse: true
                                }
                            })
                            // 获取失败
                            callback(getReturn(RETURN_CODE_BUTTON_ERROR, "createUserInfoButton 失败", {}));
                            return false;
                        }
                    })
                }
            },
            (err) => {
                callback(getReturn(RETURN_CODE_BUTTON_ERROR, "createUserInfoButton 失败", {}));
                return false;
            })
    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }

}

/**
 * 创建授权按钮 销毁接口
 */
function destoryUserInfoButton(params, callback = () => {}) {
  consolePrintWithTime("destoryUserInfoButton reciveparam", params);
  consolePrintWithTime("destoryUserInfoButton callback", callback);
  if (!isInited()) {
    callback({
        "code": RETURN_CODE_NOT_INITED,
        "message": '未完成初始化',
        "data": {}
    });
    return false;
  }
  MIAOJU_USERINFO_BUTTON.destroy();
  consolePrintWithTime("MIAOJU_USERINFO_BUTTON destoryUserInfoButton:",MIAOJU_USERINFO_BUTTON);
  callback(getSuccessReturn({}));
  
}

/**
 * 请求用户信息接口
 */
function reqUserInfo(params, callback) {
    apiRequest({
        url: MIAOJU_REQUEST_HOST[MIAOJU_ENV_CONFIG] + '/user/info',
        data: params
    }).then(
        (result) => {
            if (IS_MOCK) {
                result = {
                    "code": 1000,
                    "message": "success",
                    "data": {
                        "user_id": 10000001, //SDK用户ID
                        "platform_user_id": "lwoonwmyyyabltlwylvs", //即微信open_id
                        "platform_id": 100001, //平台ID
                        "game_id": 1001, //游戏ID
                        "token": "lwoonwmyyyabltlwylvs", //登录态
                        "nick_name": "张胖", //昵称
                        "gender": "男", //性别：男、女、未知
                        "avatar_url": "https://****", //头像
                        "union_id": "oonwmyyyabltsraslwylvs", //即微信open_id
                        "channl_id": 10000001, //渠道来源
                    }
                }
            }
            const {
                code,
                message,
                data
            } = result
            consolePrintWithTime("reqUserInfo res", code,message,data);
            if (code === 1000) {
                // 保存用户信息缓存
                wx.setStorageSync(CACHE_USER_INFO, data)
                let scopeUserInfo = true
                if(params.user_info === ""){
                  scopeUserInfo = false
                }
                callback({
                    "code": code,
                    "message": message,
                    "data": {
                        "nickName": data.nick_name || '',
                        "avatarUrl": data.avatar_url || '',
                        "unionId": data.union_id || '',
                        "createTime": data.create_time || '',
                        "gender": data.gender || '',
                        "platformUserId": data.platform_user_id || '',
                        "sdkUserId": data.user_id || 0,
                        "scopeUserInfo":scopeUserInfo
                    }
                })
            } else {
                callback({
                    "code": RETURN_CODE_USERINFO_ERROR,
                    "message": message,
                    "data": {}
                })
            }
        },
        (err) => {
          consolePrintWithTime("reqUserInfo err", err);
          callback({
              "code": RETURN_CODE_USERINFO_ERROR,
              "message": '请求 /user/info接口错误'
          })
        })
}

/**
 * 创建角色  account_id : 服id_角色id
 * @param {*} params 
 * @param {*} callback 
 */
function createRole(params, callback = () => {}) {
    if (isInited()) {
        consolePrintWithTime("createRole reciveparam", params);
        consolePrintWithTime("createRole callback", callback);
        // params = {
        //     "serverId": "11", //服ID
        //     "serverName": "传奇盛世", //服名称
        //     "roleId": "123222", //角色ID
        //     "roleName": "重生之我是吗喽111", //角色名
        //     "roleLevel": "9993", //角色等级
        //     "roleCreateTime": 1, //角色创建时间戳，秒
        //     "location": "浙江1" //大区
        // }
        // 判断参数是否正确
        if (!params.serverId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "serverId missing", {}));
            return false;
        }
        if (!params.serverName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "serverName missing", {}));
            return false;
        }
        if (!params.roleId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleId missing", {}));
            return false;
        }
        if (!params.roleName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleName missing", {}));
            return false;
        }
        if (!params.roleLevel) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleLevel missing", {}));
            return false;
        }
        if (!params.location) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "location missing", {}));
            return false;
        }
        const req_params = {
            "role_id": params.roleId,
            "role_name": params.roleName,
            "role_level": params.roleLevel,
            "server_id": params.serverId,
            "server_name": params.serverName,
            "location": params.location
        }

        apiRequest({
            url: MIAOJU_REQUEST_HOST[MIAOJU_ENV_CONFIG] + '/user/role',
            data: {
                type: 0, // 不传或0 默认创角  1表示登录角色 
                role_info: JSON.stringify(req_params) // 游戏传过来的参数转化为json字符串格式
            }
        }).then((result) => {
            if (IS_MOCK) {
                result = {
                    "code": 1000,
                    "message": "createRole success",
                    "data": {
                        "user_id": 10000001, //SDK用户ID
                        "platform_user_id": "lwoonwmyyyabltlwylvs", //即微信open_id
                        "platform_id": 100001, //平台ID
                        "game_id": 1001, //游戏ID
                        "token": "lwoonwmyyyabltlwylvs", //登录态
                        "nick_name": "张胖", //昵称
                        "gender": "男", //性别：男、女、未知
                        "avatar_url": "https://****", //头像
                        "union_id": "oonwmyyyabltsraslwylvs", //即微信open_id
                        "channl_id": 10000001, //渠道来源
                        "role_create_time": "2018-01-01 01:23:45" // 角色创建时间
                    }
                }
            }
            const {
                code,
                data
            } = result
            if (code === 1000) {
                // 缓存创角信息
                wx.setStorageSync(CACHE_ROLE_INFO, req_params);
                // 设置数数账号ID account_id : 服id_角色id
                MIAOJU_SSKJSDK.login(getAccountID(params))
                const options = getOptions(0)
                let channel_id = 0
                if (Object.keys(options.query).length !== 0) {
                    channel_id = options.query.channel_id || 0
                }
                const user_info = wx.getStorageSync(CACHE_USER_INFO)
                // 只设置一次的属性
                ssUserSet({
                    type: 2,
                    properties: {
                        mj_role_create_time: data.role_create_time || getTime(),
                        mj_role_id: params.roleId,
                        mj_passport: user_info.user_id || '',
                        mj_wx_openid: user_info.platform_user_id || '',
                        mj_rolename: params.roleName,
                        mj_server_id: params.serverId,
                        mj_location: params.location,
                        mj_channel_id: channel_id,
                        mj_version_game: MIAOJU_GAME_VERSION,
                        mj_version_sdk: MIAOJU_SDK_VERSION
                    }
                })
                // 设置用户属性
                ssUserSet({
                    type: 1,
                    properties: {
                        mj_role_name: params.roleName
                    }
                })
                // 玩家创建角色时上报
                ssReport({
                    eventName: 'mj_create_role'
                })
                // 创角 引力引擎上报
                ylyqReport({
                    eventName: 'gravity_create_role'
                })
                ylyqReport({
                  eventName: 'CreateRole',
                  properties: {
                    roleName:params.roleName
                  }
                })
                // 执行回调 callback
                callback(result);
            } else {
                callback(getReturn(RETURN_CODE_ROLE_ERROR, "createRole error", {}))
            }
        }, (err) => {
            callback(getReturn(RETURN_CODE_ROLE_ERROR, "请求 /user/role 接口错误", {}))
        })
    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }
}

/**
 * 登录角色  account_id : 服id_角色id
 * @param {*} params 
 * @param {*} callback 
 */
function loginRole(params, callback = () => {}) {
    if (isInited()) {
        consolePrintWithTime("loginRole reciveparam", params);
        consolePrintWithTime("loginRole callback", callback);
        // params = {
        //     "serverId": "12", //服ID
        //     "serverName": "传奇盛世2", //服名称
        //     "roleId": "12333", //角色ID
        //     "roleName": "重生之我是吗喽4", //角色名
        //     "roleLevel": "9994", //角色等级
        //     "roleCreateTime": 15, //角色创建时间戳，秒
        //     "location": "浙江4" //大区
        // }
        // 判断参数是否正确
        if (!params.serverId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "serverId missing", {}));
            return false;
        }
        if (!params.serverName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "serverName missing", {}));
            return false;
        }
        if (!params.roleId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleId missing", {}));
            return false;
        }
        if (!params.roleName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleName missing", {}));
            return false;
        }
        if (!params.roleLevel) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleLevel missing", {}));
            return false;
        }
        if (!params.location) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "location missing", {}));
            return false;
        }
        const req_params = {
            "role_id": params.roleId,
            "role_name": params.roleName,
            "role_level": params.roleLevel,
            "server_id": params.serverId,
            "server_name": params.serverName,
            "location": params.location
        }
        apiRequest({
            url: MIAOJU_REQUEST_HOST[MIAOJU_ENV_CONFIG] + '/user/role',
            data: {
                type: 1, // 不传或0 默认创角  1表示登录角色 
                role_info: JSON.stringify(req_params) // 游戏传过来的参数转化为json字符串格式
            }
        }).then((result) => {
            if (IS_MOCK) {
                result = {
                    "code": 1000,
                    "message": "loginRole success",
                    "data": {
                        "user_id": 10000001, //SDK用户ID
                        "platform_user_id": "lwoonwmyyyabltlwylvs", //即微信open_id
                        "is_new": 1, //是否新用户,新用户则需要调用注册reg接口
                        "nickname": "测试昵称",
                        "avatar": "http://*****",
                        "unionid": "123456",
                        "token": "abcdefg"
                    }
                }
            }
            const {
                code,
                data
            } = result
            if (code === 1000) {
                // 缓存登录角色的信息
                wx.setStorageSync(CACHE_ROLE_INFO, req_params);
                // 设置数数账号ID
                MIAOJU_SSKJSDK.login(getAccountID(params))
                // 玩家登录角色时上报
                ssReport({
                    eventName: 'mj_login_role'
                })
                // 执行回调 callback
                callback(getSuccessReturn(data));
            } else {
                callback(getReturn(RETURN_CODE_ROLE_ERROR, "loginRole error", {}))
            }
        }, (err) => {
            callback(getReturn(RETURN_CODE_ROLE_ERROR, "请求 /user/role 接口错误", {}))
        })
        // 登录角色缓存角色信息
        callback({
            "code": RETURN_CODE_SUCCESS,
            "message": "loginRole success",
            "data": {}
        });
        return false;
    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }
}

/**
 * 切换角色 
 * @param {*} params 
 * @param {*} callback 
 */
function changeRole(params, callback = () => {}) {
    if (isInited()) {
        consolePrintWithTime("changeRole reciveparam", params);
        consolePrintWithTime("changeRole callback", callback);
        // params = {
        //     "serverId": "14", //服ID
        //     "serverName": "传奇盛世3", //服名称
        //     "roleId": "123", //角色ID
        //     "roleName": "重生之我是吗喽", //角色名
        //     "roleLevel": "999", //角色等级
        //     "roleCreateTime": 1, //角色创建时间戳，秒
        //     "location": "浙江" //大区
        // }
        // 判断参数是否正确
        if (!params.serverId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "serverId missing", {}));
            return false;
        }
        if (!params.serverName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "serverName missing", {}));
            return false;
        }
        if (!params.roleId) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleId missing", {}));
            return false;
        }
        if (!params.roleName) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleName missing", {}));
            return false;
        }
        if (!params.roleLevel) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "roleLevel missing", {}));
            return false;
        }
        if (!params.location) {
            callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "location missing", {}));
            return false;
        }
        // 玩家切换角色时上报数数
        ssReport({
            eventName: 'mj_change_role'
        })
        callback({
            "code": RETURN_CODE_SUCCESS,
            "message": "changeRole success",
            "data": {}
        });
        return false;
    } else {
        // 未初始化
        callback({
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        });
    }
}

/**
 * 上报数据 废弃
 * @param {*} params 
 * @param {*} callback 
 */
function reportLog(params, callback = () => {}) {
    consolePrintWithTime("reportLog reciveparam", params);
    consolePrintWithTime("reportLog callback", callback);
    callback({
        code: RETURN_CODE_SUCCESS,
        message: "reportLog成功",
        data: {
            "clint_id": "xertertertertertert",
            "isme": "yes",
            "newadd": 1
        }
    });
}

/**
 * 上报数数 自定义事件
 * @param {*} params 
 * @param {*} callback 
 */
function ssReport(params, callback = () => {}) {
    consolePrintWithTime("ssReport reciveparam", params);
    consolePrintWithTime("ssReport callback", callback);
    // params = {
    //     eventName: '',
    //     properties: {}
    // }

    // 判断参数是否正确
    if (!params.eventName) {
        callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "eventName missing", {}));
        return false;
    }
    // 发送事件
    MIAOJU_SSKJSDK.track({
        eventName: params.eventName, //追踪事件的名称
        properties: params.properties || {}, //需要上传的事件属性
    });
    callback({
        code: RETURN_CODE_SUCCESS,
        message: "ssReport成功",
        data: {}
    });
}

/**
 * 上报数数 设置用户属性
 * @param {*} params  type 设置用户属性的类型 1 user_set 2 userSetOnce 3 userAdd 
 * @param {*} callback 
 */
function ssUserSet(params, callback = () => {}) {
    consolePrintWithTime("ssUserSet reciveparam", params);
    consolePrintWithTime("ssUserSet callback", callback);
    // params = {
    //     type: 1, // 设置用户属性的类型 1 user_set 2 userSetOnce 3 userAdd 
    //     properties: {}
    // }
    if (!params.type) {
        callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "type missing", {}));
        return false;
    }
    if (!params.properties) {
        callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "properties missing", {}));
        return false;
    }
    if (Object.keys(params.properties).length == 0) {
        // 说明params.properties为空
        callback(getReturn(RETURN_CODE_PARAMS_ERROR, "properties error", {}));
        return false;
    }

    if (params.type == 1) {
        // 设置用户属性
        MIAOJU_SSKJSDK.userSet({
            properties: params.properties //事件属性
        })
    } else if (params.type == 2) {
        // 用户属性只要设置一次
        MIAOJU_SSKJSDK.userSetOnce({
            properties: params.properties //事件属性
        })
    } else if (params.type == 3) {
        // 用户属性累加
        MIAOJU_SSKJSDK.userAdd({
            properties: params.properties //事件属性
        })
    }
    callback({
        code: RETURN_CODE_SUCCESS,
        message: "ssUserSet成功",
        data: {}
    });
}

/**
 * 上报引力引擎 自定义事件
 * @param {*} params 
 * @param {*} callback 
 */
function ylyqReport(params, callback = () => {}) {
    consolePrintWithTime("ylyqReport reciveparam", params);
    consolePrintWithTime("ylyqReport callback", callback);
    // params = {
    //     eventName: '',
    //     properties: {}
    // }
    if (!params.eventName) {
        callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "eventName missing", {}));
        return false;
    }
    if(params.eventName == 'Pay'){
      //支付
      MIAOJU_YLYQSDK.onPayEvent(params.properties.payAmount, params.properties.payType, params.properties.orderId, params.properties.payReason, params.properties.payMethod);
    }else if(params.eventName == 'AddToWishList'){
      //加入收藏
      MIAOJU_YLYQSDK.onAddToWishListEvent("default")
    }else if(params.eventName == 'Share'){
      //分享
      if(params.properties.type == 'APP_MESSAGE'){
        MIAOJU_YLYQSDK.onShareEvent("APP_MESSAGE");
      }else{
        MIAOJU_YLYQSDK.onShareEvent("TIME_LINE");
      }
    }else if(params.eventName == 'AddToWishList'){
      //加入收藏
      MIAOJU_YLYQSDK.onAddToWishListEvent("my")
    }else if(params.eventName == 'CreateRole'){
      //角色上报
      MIAOJU_YLYQSDK.onCreateRoleEvent(params.properties.roleName);
    }else if(params.eventName == 'UpdateLevel'){
      //等级提升
      MIAOJU_YLYQSDK.onUpdateLevelEvent(params.properties.roleLevel);
    }else if(params.eventName == 'ViewMallContent'){
      //用户在小游戏内浏览商城页面时上报。
      MIAOJU_YLYQSDK.onViewMallContentEvent();
    }else if(params.eventName == 'ViewActivityContent'){
      //用户在小游戏内浏览活动页面时上报。
      MIAOJU_YLYQSDK.onViewActivityContentEvent();
    }else if(params.eventName == 'TutorialFinish'){
      //在用户完成游戏新手指引教程或者完成教程关卡后上报 TUTORIAL_FINISH 行为。
      MIAOJU_YLYQSDK.onTutorialFinishEvent();
    }else{
      MIAOJU_YLYQSDK.track(
        params.eventName, //追踪事件的名称
        params.properties || {}, //需要上传的事件属性
      );
    }

    
    callback({
        code: RETURN_CODE_SUCCESS,
        message: "ylyqReport成功",
        data: {}
    });
}

/**
 * 获取系统参数及启动参数
 */
function getUserLaunchInfo() {
    if (isInited()) {
        const options = getOptions(0)
        return {
            code: RETURN_CODE_SUCCESS,
            message: "getUserLaunchInfo成功",
            data: {
                "launchOptions": options,
                "systemInfo": MIAOJU_SYSTEMINFO
            }
        }
    } else {
        // 未初始化
        return {
            "code": RETURN_CODE_NOT_INITED,
            "message": '未完成初始化',
            "data": {}
        };
    }
}

/**
 * 加入语音房间
 * @param {*} params 参数
 * @param {*} callback 调用方法回调
 * @param {*} callbackListener 监听回调
 */
function joinVoIPChat(params, callback = () => {}, callbackListener = () => {}){
  const funcNameMsg = 'joinVoIPChat';
  if (!isInited()) {
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": funcNameMsg+'未完成初始化',
      "data": {}
    });
    return;
  }
  consolePrintWithTime(funcNameMsg+" reciveparam", params,callback,callbackListener);

  wx.getSetting({
    success(res) {
      consolePrintWithTime(funcNameMsg+" getSetting success:",res)
      if (!res.authSetting['scope.record']) {
        consolePrintWithTime(funcNameMsg+" getSetting no scope.record:")
        wx.authorize({
          scope: 'scope.record',
          success () {
            consolePrintWithTime(funcNameMsg+" getSetting authorize suc:")
            joinVoIPChatSDK(params,callback,callbackListener)
          },
          fail(){
            consolePrintWithTime(funcNameMsg+" getSetting authorize fail:")
            joinVoIPChatSDK(params,callback,callbackListener)
          }
        })
      }else{
        joinVoIPChatSDK(params,callback,callbackListener)
      }
    }
  })
}

/**
 * 调用语音聊天接口
 * @param {*} params 
 * @param {*} callback 
 */
function joinVoIPChatSDK(params,callback = () => {}, callbackListener = () => {}){
  consolePrintWithTime("joinVoIPChatSDK start",params,callback)
  if (!params.groupId) {
    callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "joinVoIPChatSDK groupId missing", {}));
    return false;
  }
  if (!params.muteConfig) {
    callback(getReturn(RETURN_CODE_PARAMS_EMPTY, "joinVoIPChatSDK muteConfig missing", {}));
    return false;
  }

  //调用接口获取语音房间的加密参数
  apiRequest({
    url: MIAOJU_REQUEST_HOST[MIAOJU_ENV_CONFIG] + '/media/voipsign',
    data: {"group_id":params.groupId}
  }).then(
    (result) => {
        const {
            code,
            message,
            data
        } = result;
        consolePrintWithTime("joinVoIPChatSDK APIRES:",result)
        if (code === 1000) {
          //接口成功，拉起语音聊天房间
          wx.joinVoIPChat({
            roomType: "voice",
            signature: data.signature,
            nonceStr: data.nonce_str,
            timeStamp: data.time_stamp,
            groupId: params.groupId,
            muteConfig:params.muteConfig,
            forceCellularNetwork:true,
            success(res) {
              consolePrintWithTime("joinVoIPChatSDK success:",res)
        
              //监听成员通话状态变化
              wx.onVoIPChatSpeakersChanged(listenRes => {
                consolePrintWithTime("onVoIPChatSpeakersChanged success:",listenRes)
                callbackListener(getSuccessReturn({"type":"onVoIPChatSpeakersChanged","openIdList":res.openIdList}))
              });
              //监听成员在线状态变化
              wx.onVoIPChatMembersChanged(listenRes => {
                consolePrintWithTime("onVoIPChatMembersChanged success:",listenRes)
                callbackListener(getSuccessReturn({"type":"onVoIPChatMembersChanged","openIdList":res.openIdList}))
              });

              //数数上报
              ssReport({
                eventName: 'mj_yp_phone',
                properties: {
                  mj_yp_type: 'success',
                  mj_yp_group_id: params.groupId,
                  mj_yp_reason: ''
                }
              })
              callback(getSuccessReturn({"openIdList":res.openIdList}));
            },
            fail(res) {
              consolePrintWithTime("joinVoIPChatSDK fail:",res);
              //数数上报
              ssReport({
                eventName: 'mj_yp_phone',
                properties: {
                  mj_yp_type: 'fail',
                  mj_yp_group_id: params.groupId,
                  mj_yp_reason: res.errCode+'###'+res.errMsg
                }
              })
              callback(getReturn(RETURN_CODE_SYSTEM_ERROR, "joinVoIPChatSDK fail", res));
            },
            complete(res) {
              consolePrintWithTime("joinVoIPChatSDK complete:",res);
            }
          })
        } else {
          consolePrintWithTime("joinVoIPChatSDK API CODEERROR:",result);
          callback(getReturn(code, message, {}));
        }
  }, (err) => {
    consolePrintWithTime("joinVoIPChatSDK API ERROR:",err);
    callback(getReturn(RETURN_CODE_SYSTEM_ERROR, 'joinVoIPChatSDK API ERROR', {}));
  })
}

/**
 * 退出语音房间
 * @param {*} params 
 * @param {*} callback 
 */
function exitVoIPChat(params, callback = () => {}){
  const funcNameMsg = 'exitVoIPChat';
  if (!isInited()) {
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": funcNameMsg+'未完成初始化',
      "data": {}
    });
    return;
  }
  consolePrintWithTime(funcNameMsg+" reciveparam", params,callback);
  wx.exitVoIPChat({
    success(res) {
      consolePrintWithTime(funcNameMsg+" success:",res);
      callback(getSuccessReturn({}));
    },
    fail(res) {
      consolePrintWithTime(funcNameMsg+" fail:",res);
      callback(getReturn(RETURN_CODE_SYSTEM_ERROR, funcNameMsg+" fail", res));
    },
    complete(res) {
      consolePrintWithTime(funcNameMsg+" complete:",res);
    }
  });
}

/**
 * 更新实时语音静音设置
 * @param {*} params 
 * @param {*} callback 
 */
function updateVoIPChatMuteConfig(params, callback = () => {}){
  const funcNameMsg = 'updateVoIPChatMuteConfig';
  if (!isInited()) {
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": funcNameMsg+'未完成初始化',
      "data": {}
    });
    return;
  }
  consolePrintWithTime(funcNameMsg+" reciveparam", params,callback);
  if (!params.muteConfig) {
    callback(getReturn(RETURN_CODE_PARAMS_EMPTY, funcNameMsg+"muteConfig missing", {}));
    return false;
  }
  wx.updateVoIPChatMuteConfig({
    muteConfig:params.muteConfig,
    success(res) {
      consolePrintWithTime(funcNameMsg+" success:",res);
      callback(getSuccessReturn({}));
    },
    fail(res) {
      consolePrintWithTime(funcNameMsg+" fail:",res);
      callback(getReturn(RETURN_CODE_SYSTEM_ERROR, funcNameMsg+" fail", res));
    },
    complete(res) {
      consolePrintWithTime(funcNameMsg+" complete:",res);
    }
  });
}

function getLocation(params, callback = () => {}){
  const funcNameMsg = 'getLocation';
  if (!isInited()) {
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": funcNameMsg+'未完成初始化',
      "data": {}
    });
    return;
  }
  consolePrintWithTime(funcNameMsg+" reciveparam", params,callback);
  wx.getSetting({
    success(res) {
      consolePrintWithTime(funcNameMsg+" getSetting success:",res)
      if (!res.authSetting['scope.userFuzzyLocation']) {
        consolePrintWithTime(funcNameMsg+" getSetting no scope.userFuzzyLocation:")
        wx.authorize({
          scope: 'scope.userFuzzyLocation',
          success () {
            consolePrintWithTime(funcNameMsg+" getSetting authorize suc:")
            getLocationSDK(params,callback)
          }
        })
      }else{
        getLocationSDK(params,callback)
      }
    }
  })
  
}

/**
 * 调用微信的获取位置方法
 * @param {*} params 
 * @param {*} callback 
 */
function getLocationSDK(params, callback = () => {}){
  consolePrintWithTime("getLocationSDK start",params,callback)
  //调用微信方法获取坐标位置
  wx.getFuzzyLocation({
    type: 'gcj02',
    success (res) {
      consolePrintWithTime("getLocationSDK success:",res);
      const latitude = res.latitude
      const longitude = res.longitude
      //调用接口解析对应的位置信息
      apiRequest({
        url: MIAOJU_REQUEST_HOST[MIAOJU_ENV_CONFIG] + '/tools/getlocation',
        data: {"longitude":longitude,"latitude":latitude}
      }).then(
        (result) => {
            const {
                code,
                message,
                data
            } = result;
            consolePrintWithTime("getLocationSDK APIRES:",result)
            if (code === 1000) {
              callback(getSuccessReturn({
                "latitude":latitude,
                "longitude":longitude,
                "country":data.address.nation,
                "province":data.address.province,
                "city":data.address.city
              }));
            } else {
              consolePrintWithTime("getLocationSDK API CODEERROR:",result);
              callback(getReturn(code, message, {}));
            }
      }, (err) => {
        consolePrintWithTime("getLocationSDK API ERROR:",err);
        callback(getReturn(RETURN_CODE_SYSTEM_ERROR, 'getLocationSDK API ERROR', {}));
      })
    }
  });
}

/**
 * 微信一次性订阅消息
 * @param {*} params 
 * @param {*} callback 
 */
function requestSubscribeMessage(params, callback = () => {}){
  const funcNameMsg = 'requestSubscribeMessage';
  if (!isInited()) {
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": funcNameMsg+'未完成初始化',
      "data": {}
    });
    return;
  }
  consolePrintWithTime(funcNameMsg+" reciveparam", params,callback);
  if (!params.tmplIds || params.tmplIds.length === 0) {
    callback(getReturn(RETURN_CODE_PARAMS_EMPTY, funcNameMsg+" tmplIds missing", {}));
    return false;
  }

  wx.requestSubscribeMessage({
    tmplIds: params.tmplIds,//["6doVBAYuVp-yIavDyyBDVCjhltUD7LFwOfFXF8TOQ4g","m-68uQS42Bj-hKZGOksf-2BpwzMcnRclOELDkYlH50g"],
    success (res) {
      consolePrintWithTime(funcNameMsg+" suc:",res);
      let data = {}
      const keys = Object.keys(res)
      keys.forEach(function (item,key) {
        // consolePrintWithTime("requestSubscribeMessage keys:",item,key);
        if(item != "errMsg"){
          data[item] = res[item]
          //数数上报
          ssReport({
            eventName: 'mj_kt_onexinxi',
            properties: {
              mj_tmpl_id: item,
              mj_msg_action: res[item]
            }
          })
        }
      })
      callback(getSuccessReturn(data));
    },
    fail (res) {
      consolePrintWithTime(funcNameMsg+" fail:",res);
      callback(getReturn(RETURN_CODE_SYSTEM_ERROR, funcNameMsg+" fail", res));
    }
  })
  
}

/**
 * 微信版本更新订阅消息
 * @param {*} params 
 * @param {*} callback 
 */
function requestSubscribeSystemMessage(params, callback = () => {}){
  const funcNameMsg = 'requestSubscribeSystemMessage';
  if (!isInited()) {
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": funcNameMsg+'未完成初始化',
      "data": {}
    });
    return;
  }
  consolePrintWithTime(funcNameMsg+" reciveparam", params,callback);
  if (!params.msgTypeList || params.msgTypeList.length === 0) {
    callback(getReturn(RETURN_CODE_PARAMS_EMPTY, funcNameMsg+" msgTypeList missing", {}));
    return false;
  }
  wx.requestSubscribeSystemMessage({
    msgTypeList: params.msgTypeList,//['SYS_MSG_TYPE_WHATS_NEW', 'SYS_MSG_TYPE_INTERACTIVE', 'SYS_MSG_TYPE_RANK'],
    success (res) {
      consolePrintWithTime(funcNameMsg+" wx.requestSubscribeSystemMessage res", res);
      let data = {}
      const keys = Object.keys(res)
      keys.forEach(function (item,key) {
        // consolePrintWithTime("requestSubscribeMessage keys:",item,key);
        if(item != "errMsg"){
          data[item] = res[item]
          //数数上报
          if(item === 'SYS_MSG_TYPE_WHATS_NEW'){
            ssReport({
              eventName: 'mj_kt_gengxinxinxi',
              properties: {
                mj_tmpl_id: item,
                mj_msg_action: res[item]
              }
            })
          }else{
            ssReport({
              eventName: 'mj_kt_dingyuexinxi',
              properties: {
                mj_tmpl_id: item,
                mj_msg_action: res[item]
              }
            })
          }
        }
      })
      callback(getSuccessReturn(data));
    },
    fail (res) {
      consolePrintWithTime(funcNameMsg+" fail:",res);
      callback(getReturn(RETURN_CODE_SYSTEM_ERROR, funcNameMsg+" fail", res));
    },
    complete (res) {
      consolePrintWithTime(funcNameMsg+" complete:",res);
    }
  })
}

/**
 * 检查小游戏是否加入收藏
 * @param {*} params 
 * @param {*} callback 
 */
function checkIsAddedToMyMiniProgram(params, callback = () => {}){
  const funcNameMsg = 'checkIsAddedToMyMiniProgram';
  if (!isInited()) {
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": funcNameMsg+'未完成初始化',
      "data": {}
    });
    return;
  }
  consolePrintWithTime(funcNameMsg+" reciveparam", params,callback);
  wx.checkIsAddedToMyMiniProgram({
    success (res) {
      consolePrintWithTime(funcNameMsg+" wx.checkIsAddedToMyMiniProgram res", res);
      if(res.added === true){
        ssReport({
          eventName: 'mj_sc_game'
        })
        //上报引力引擎，引力上报腾讯
        ylyqReport({
          eventName: 'AddToWishList'
        })
      }
      callback(getSuccessReturn(res));
    },
    fail (res) {
      consolePrintWithTime(funcNameMsg+" fail:",res);
      callback(getReturn(RETURN_CODE_SYSTEM_ERROR, funcNameMsg+" fail", res));
    },
    complete (res) {
      consolePrintWithTime(funcNameMsg+" complete:",res);
    }
  });
}

/**
 * 获取用户设置权限信息
 * @param {*} params 
 * @param {*} callback 
 */
function getSetting(params, callback = () => {}){
  const funcNameMsg = 'getSetting';
  if (!isInited()) {
    callback({
      "code": RETURN_CODE_NOT_INITED,
      "message": funcNameMsg+'未完成初始化',
      "data": {}
    });
    return;
  }
  consolePrintWithTime(funcNameMsg+" reciveparam", params,callback);

  wx.getSetting({
    withSubscriptions: true,
    success(res) {
      consolePrintWithTime(funcNameMsg+" wx.getSetting res", res);
      callback(getSuccessReturn(res));
    },
    fail(res){
      consolePrintWithTime(funcNameMsg+" fail:",res);
      callback(getReturn(RETURN_CODE_SYSTEM_ERROR, funcNameMsg+" fail", res));
    },
    complete(res){
      consolePrintWithTime(funcNameMsg+" complete:",res);
    }
  })
}

function buttonOne(params, callback) {
    consolePrintWithTime("buttonOne reciveparam", params);
    consolePrintWithTime("buttonOne callback", callback);
    /*
    wx.authorize({
      scope: 'scope.record',//
      success () {
        consolePrintWithTime("joinVoIPChat getSetting authorize suc:")
      }
    });*/

    wx.getSetting({
      withSubscriptions: true,
      success(res) {
        consolePrintWithTime("joinVoIPChat getSetting success:",res)
        if (!res.authSetting['scope.record']) {
          consolePrintWithTime("joinVoIPChat getSetting no scope.record:")
          wx.authorize({
            scope: 'scope.record',
            success () {
              consolePrintWithTime("joinVoIPChat getSetting authorize suc:")
              joinVoIPChatSDK(params,callback)
            }
          })
        }else{
          joinVoIPChatSDK(params,callback)
        }
      }
    })
    
    
    callback({
        code: RETURN_CODE_SUCCESS,
        message: "buttonOne成功",
        data: {
            "clint_id": "xertertertertertert111",
            "isme": "yes",
            "newadd": 1
        }
    });
}

function buttonTwo(params, callback) {
    consolePrintWithTime("buttonTwo reciveparam", params);
    consolePrintWithTime("buttonTwo callback", callback);
    
    // 一次性订阅
    /*
    wx.requestSubscribeMessage({
      tmplIds: ['m-68uQS42Bj-hKZGOksf-2BpwzMcnRclOELDkYlH50g'],
      success (res) {
        consolePrintWithTime("buttonTwo res:",res)
        
      }
    })*/

    //系统订阅消息 需要真机测试
    /*
    wx.requestSubscribeSystemMessage({
        msgTypeList: ['SYS_MSG_TYPE_INTERACTIVE', 'SYS_MSG_TYPE_RANK'],
        success(res) {
          consolePrintWithTime("buttonTwo res:",res)
          // {
          //      errMsg: "requestSubscribeSystemMessage:ok",
          //      SYS_MSG_TYPE_INTERACTIVE: 'accept'
          //      SYS_MSG_TYPE_RANK: 'reject'
          // }
        },
        fail(res) {
          console.log(res)
        },
        complete(res) {
          console.log(res)
    } })
    */
  
    //更新提醒消息 需要真机测试
    /*
    wx.requestSubscribeSystemMessage({
      msgTypeList: ['SYS_MSG_TYPE_WHATS_NEW'],
      success (res) {
        consolePrintWithTime("buttonTwo res:",res)
      }
    });*/

    //检查是否加入到收藏
    wx.checkIsAddedToMyMiniProgram({
      success (res) {
        consolePrintWithTime("buttonTwo res:",res)
      },
      fail (res) {
        consolePrintWithTime("buttonTwo fail res:",res)
      }
    });

    //上报新手引导完成
    ylyqReport({
      eventName: 'TutorialFinish'
    })


    callback({
        code: RETURN_CODE_SUCCESS,
        message: "buttonTwo成功",
        data: {
            "clint_id": "xertertertertertert",
            "isme": "yes",
            "newadd": 1
        }
    });
}

/**
 * 判断是否初始化
 */
function isInited() {
    return MIAOJU_IS_INITED
}

/**
 * 获得数数登录账号id
 */
function getAccountID(params) {
    return params.serverId + '_' + params.roleId
}

/**
 * 获取当前时间字符串
 * @param {} args 
 */
function getTime() {
    const date = new Date(); // 获取当前时间的 Date 对象

    const year = date.getFullYear();
    const month = String(date.getMonth() + 1).padStart(2, '0'); // 月份从0开始，需要+1，然后补零
    const day = String(date.getDate()).padStart(2, '0');
    const hours = String(date.getHours()).padStart(2, '0');
    const minutes = String(date.getMinutes()).padStart(2, '0');
    const seconds = String(date.getSeconds()).padStart(2, '0');

    const formattedDateTime = `${year}-${month}-${day} ${hours}:${minutes}:${seconds}`;
    return formattedDateTime
}

/**
 * wx.request请求接口
 * @param {*} args 
 */
function apiRequest(args = {}) {
    const user_info = wx.getStorageSync(CACHE_USER_INFO)
    const role_info = wx.getStorageSync(CACHE_ROLE_INFO)

    const options = getOptions(0)
    args.url = args.url
    args.data = args.data || {}
    // 公共参数
    args.data.platform_id = MIAOJU_PLATFORM_ID
    args.data.game_id = MIAOJU_GAME_ID
    args.data.client_id = getClientId()
    if (!args.data.token) {
      if(args.url.includes("user/login") && args.data.code){
        //如果是登录请求且参数里包含code，则不带token
      }else{
        args.data.token = user_info.token || ''
      }      
    }
    // 如果存在role_info 的缓存，则带上
    if (!args.data.role_info && role_info) {
        args.data.role_info = JSON.stringify(role_info) || ''
    }
    if (Object.keys(options.query).length !== 0) {
        // 说明query有值
        args.data.query = JSON.stringify(options.query)
        args.data.channel_id = options.query.channel_id || ''
    }
    args.data.device_info = JSON.stringify(MIAOJU_DEVICEINFO)
    consolePrintWithTime("请求链接:", args.url, args.data);

    return new Promise((resolve, reject) => {
        wx.request({
            fail: function (err) {
                consolePrintWithTime("wx.request 请求接口出错：", err);
                reject(err)
            },
            success: function (res) {
                // 对请求接口统一处理
                if (res.statusCode == 200) {
                    const code = res.data.code
                    switch (code) {
                        default:
                            resolve(res.data)
                            break;
                    }
                }
            },
            ...args,
        })
    })
}

/**
 * wx异步方法封装
 * @param {*} api 
 */
function wxPromisify(api) {
    return (args = {}) => {
        return new Promise((resolve, reject) => {
            api({
                fail: function (err) {
                    consolePrintWithTime("wx调用错误：", err);
                    reject(err)
                },
                success: function (res) {
                    consolePrintWithTime("wx调用成功：", res);
                    resolve(res)
                },
                ...args,
            })
        })
    }
}

/**
 * 获取启动参数
 * @isCache 1取全局变量，0取系统函数
 */
function getOptions(isCache = 0) {
    let options = {};
    if (isCache == 1) {
        options = MIAOJU_LAUNCHOPTIONS;
    } else {
        options = wx.getLaunchOptionsSync();
        MIAOJU_LAUNCHOPTIONS = options;
    }
    return options;
}

/**
 * 获取客户端id
 */
function getClientId() {
    var clientId = ''
    try {
        var value = wx.getStorageSync(CACHE_CLIENT_ID)
        if (value) {
            clientId = value;
        }
    } catch (e) {}

    if (clientId == '') {
        let obj = new GUID();
        clientId = obj.newGUID(MIAOJU_GAME_ID);
        try {
            wx.setStorageSync(CACHE_CLIENT_ID, clientId)
        } catch (e) {}
    }

    return clientId;

}

//生成guid
function GUID() {
    this.date = new Date();

    /* 判断是否初始化过，如果初始化过以下代码，则以下代码将不再执行，实际中只执行一次 */
    if (typeof this.newGUID != 'function') {

        /* 生成GUID码 */
        GUID.prototype.newGUID = function (str = "") {
            this.date = new Date();
            var guidStr = '';
            var sexadecimalDate = this.hexadecimal(this.getGUIDDate(), 16);
            var sexadecimalTime = this.hexadecimal(this.getGUIDTime(), 16);
            for (var i = 0; i < 9; i++) {
                guidStr += Math.floor(Math.random() * 16).toString(16);
            }
            guidStr += sexadecimalDate;
            guidStr += sexadecimalTime;
            while (guidStr.length < 32) {
                guidStr += Math.floor(Math.random() * 16).toString(16);
            }

            var strGuid = str + '-' + this.formatGUID(guidStr)
            return strGuid;
        }

        /*
         * 功能：获取当前日期的GUID格式，即8位数的日期：19700101
         * 返回值：返回GUID日期格式的字条串
         */
        GUID.prototype.getGUIDDate = function () {
            return this.date.getFullYear() + this.addZero(this.date.getMonth() + 1) + this.addZero(this.date.getDay());
        }

        /*
         * 功能：获取当前时间的GUID格式，即8位数的时间，包括毫秒，毫秒为2位数：12300933
         * 返回值：返回GUID日期格式的字条串
         */
        GUID.prototype.getGUIDTime = function () {
            return this.addZero(this.date.getHours()) + this.addZero(this.date.getMinutes()) + this.addZero(this.date.getSeconds()) + this.addZero(parseInt(this.date.getMilliseconds() / 10));
        }

        /*
         * 功能: 为一位数的正整数前面添加0，如果是可以转成非NaN数字的字符串也可以实现
         * 参数: 参数表示准备再前面添加0的数字或可以转换成数字的字符串
         * 返回值: 如果符合条件，返回添加0后的字条串类型，否则返回自身的字符串
         */
        GUID.prototype.addZero = function (num) {
            if (Number(num).toString() != 'NaN' && num >= 0 && num < 10) {
                return '0' + Math.floor(num);
            } else {
                return num.toString();
            }
        }

        /*
         * 功能：将y进制的数值，转换为x进制的数值
         * 参数：第1个参数表示欲转换的数值；第2个参数表示欲转换的进制；第3个参数可选，表示当前的进制数，如不写则为10
         * 返回值：返回转换后的字符串
         */
        GUID.prototype.hexadecimal = function (num, x, y) {
            if (y != undefined) {
                return parseInt(num.toString(), y).toString(x);
            } else {
                return parseInt(num.toString()).toString(x);
            }
        }

        /*
         * 功能：格式化32位的字符串为GUID模式的字符串
         * 参数：第1个参数表示32位的字符串
         * 返回值：标准GUID格式的字符串
         */
        GUID.prototype.formatGUID = function (guidStr) {
            var str1 = guidStr.slice(0, 8) + '-',
                str2 = guidStr.slice(8, 16) + '-',
                str3 = guidStr.slice(16, 24) + '-',
                str4 = guidStr.slice(24);
            return str1 + str2 + str3 + str4;
        }
    }
}

/**
 * 输出日志带日期时间
 */
function consolePrintWithTime(...msgs) {
    const now = new Date();
    const formattedDate = `${now.getFullYear()}-${(now.getMonth() + 1).toString().padStart(2, '0')}-${now.getDate().toString().padStart(2, '0')} ${now.getHours().toString().padStart(2, '0')}:${now.getMinutes().toString().padStart(2, '0')}:${now.getSeconds().toString().padStart(2, '0')}.${now.getMilliseconds().toString().padStart(3, '0')}`;
    console.log("[", formattedDate, "] MJSDK:", ...msgs);
}
/**
 * 成功返回值
 */
function getSuccessReturn(data) {
    consolePrintWithTime("getSuccessReturn:", data);
    return {
        code: RETURN_CODE_SUCCESS,
        message: "",
        data: data
    }
}
/**
 * 返回值
 */
function getReturn(code, message, data) {
    consolePrintWithTime("getReturn:", code, message, data);
    return {
        code: code,
        message: message,
        data: data
    }
}

const MiaojuSdk = {
    init: init,
    pay: pay,
    rewardVideoAdCreate: rewardVideoAdCreate,
    rewardVideoAdCreate2: rewardVideoAdCreate,
    rewardVideoAdShow: rewardVideoAdShow,
    share: share,
    shareOpen: shareOpen,
    openCustomerService: openCustomerService,
    getUserInfo: getUserInfo,
    createUserInfoButton: createUserInfoButton,
    destoryUserInfoButton:destoryUserInfoButton,
    createRole: createRole,
    loginRole: loginRole,
    changeRole: changeRole,
    reportLog: reportLog,
    ssReport: ssReport,
    ssUserSet: ssUserSet,
    ylyqReport: ylyqReport,
    getUserLaunchInfo: getUserLaunchInfo,
    joinVoIPChat:joinVoIPChat,
    exitVoIPChat:exitVoIPChat,
    updateVoIPChatMuteConfig:updateVoIPChatMuteConfig,
    getLocation:getLocation,
    getSetting:getSetting,
    requestSubscribeMessage:requestSubscribeMessage,
    requestSubscribeSystemMessage:requestSubscribeSystemMessage,
    checkIsAddedToMyMiniProgram:checkIsAddedToMyMiniProgram,
    buttonOne: buttonOne,
    buttonTwo: buttonTwo
};

//module.exports = MiaojuSdk;
export default MiaojuSdk;